#include <Wire.h>
#include <SPI.h>
#include <PCA9557.h>
#include <lvgl.h>
#include "ui.h"
#include "gfx_conf.h"
#include <WiFi.h>
#include <Arduino.h>
#include "ui_events.c"
#include <esp_now.h>

uint8_t broadcastAddress[] = {0x24, 0xdc, 0xc3, 0xa0, 0x40, 0x74};

// Structure example to send data
// Must match the receiver structure
typedef struct {
    float Slider1;
    float Slider2;  
    int BTN_Cut_Left1;//not working
    int BTN_Cut_Left2;//not working 
    int AdjustUP;//test
    int UP;//test
    int Select1;//test
} struct2_message2;

// Create a struct_message called myData
struct2_message2 myData2;

// Create peer interface
esp_now_peer_info_t peerInfo;

// callback when data is sent
void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  //Serial.print("\r\nLast Packet Send Status:\t");
  //Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
}

static lv_disp_draw_buf_t draw_buf;
static lv_color_t disp_draw_buf1[screenWidth * screenHeight / 8];
static lv_color_t disp_draw_buf2[screenWidth * screenHeight / 8];
static lv_disp_drv_t disp_drv;

PCA9557 Out; // for touch timing init

/* Display flushing */
void my_disp_flush(lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p) {
    uint32_t w = (area->x2 - area->x1 + 1);
    uint32_t h = (area->y2 - area->y1 + 1);

    tft.pushImageDMA(area->x1, area->y1, w, h, (lgfx::rgb565_t*)&color_p->full);

    lv_disp_flush_ready(disp);
}

void my_touchpad_read(lv_indev_drv_t *indev_driver, lv_indev_data_t *data) {
    uint16_t touchX, touchY;
    bool touched = tft.getTouch(&touchX, &touchY);
    if (!touched) {
        data->state = LV_INDEV_STATE_REL;
    } else {
        data->state = LV_INDEV_STATE_PR;

        // Set the coordinates
        data->point.x = touchX;
        data->point.y = touchY;

       // Serial.print("Data x ");
       // Serial.println(touchX);

        //Serial.print("Data y ");
        //Serial.println(touchY);
    }
}

void setup() {
    Serial.begin(9600);

    // Set device as a Wi-Fi Station
    WiFi.mode(WIFI_STA);

    // Init ESP-NOW
    if (esp_now_init() != ESP_OK) {
       // Serial.println("Error initializing ESP-NOW");
        return;
    }

    // Register Send CB to get the status of transmitted packet
    esp_now_register_send_cb(OnDataSent);

    // Register peer
    memcpy(peerInfo.peer_addr, broadcastAddress, 6);
    peerInfo.channel = 0;  
    peerInfo.encrypt = false;
    
    // Add peer        
    if (esp_now_add_peer(&peerInfo) != ESP_OK) {
       // Serial.println("Failed to add peer");
        return;
    }

    // GPIO init
#if defined (CrowPanel_50) || defined (CrowPanel_70)
    pinMode(38, OUTPUT);
    digitalWrite(38, LOW);
    pinMode(17, OUTPUT);
    digitalWrite(17, LOW);
    pinMode(18, OUTPUT);
    digitalWrite(18, LOW);
    pinMode(42, OUTPUT);
    digitalWrite(42, LOW);

    // touch timing init
    // Wire.begin(19, 20);
    // Out.reset();
    // Out.setMode(IO_OUTPUT);
    // Out.setState(IO0, IO_LOW);
    // Out.setState(IO1, IO_LOW);
    // delay(20);
    // Out.setState(IO0, IO_HIGH);
    // delay(100);
    // Out.setMode(IO1, IO_INPUT);
#elif defined (CrowPanel_43)
    pinMode(20, OUTPUT);
    digitalWrite(20, LOW);
    pinMode(19, OUTPUT);
    digitalWrite(19, LOW);
    pinMode(35, OUTPUT);
    digitalWrite(35, LOW);
    pinMode(38, OUTPUT);
    digitalWrite(38, LOW);
    pinMode(0, OUTPUT); // TOUCH-CS
#endif

    // Display Prepare
    tft.begin();
    tft.setRotation(1);
    tft.fillScreen(TFT_BLACK);
    tft.setTextSize(2);
    delay(200);

    lv_init();
    delay(100);

    lv_disp_draw_buf_init(&draw_buf, disp_draw_buf1, disp_draw_buf2, screenWidth * screenHeight / 8);
    // Initialize the display
    lv_disp_drv_init(&disp_drv);
    // Change the following line to your display resolution
    disp_drv.hor_res = screenHeight;
    disp_drv.ver_res = screenWidth;
    disp_drv.flush_cb = my_disp_flush;
    disp_drv.full_refresh = 1;
    disp_drv.draw_buf = &draw_buf;
    lv_disp_drv_register(&disp_drv);

    // Initialize the (dummy) input device driver
    static lv_indev_drv_t indev_drv;
    lv_indev_drv_init(&indev_drv);
    indev_drv.type = LV_INDEV_TYPE_POINTER;
    indev_drv.read_cb = my_touchpad_read;
    lv_indev_drv_register(&indev_drv);

    tft.fillScreen(TFT_BLACK);

    // Please do not use LVGL Demo and UI export from Squareline Studio at the same time.
    // lv_demo_widgets(); // LVGL demo
    ui_init(); // UI function that comes from ui.c

    Serial.println("Setup done");
}

void loop() {
    myData2.Slider1 = lv_slider_get_value(ui_Slider1);
    myData2.Slider2 = lv_slider_get_value(ui_Slider2);
    myData2.BTN_Cut_Left2 = lv_obj_has_state(ui_BTN_Cut_Left2, LV_STATE_CHECKED) ? 1 : 0;
    myData2.BTN_Cut_Left2 =lv_event_get_code(ui_BTN_Cut_Left2);
    // Send message via ESP-NOW
    esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *) &myData2, sizeof(myData2));
   
    if (result == ESP_OK) {
        Serial.println("Sent with success");
    } else {
        Serial.println("Error sending the data");
    }
    delay(50);

    Serial.print("BTN_Cut_Left2 State: ");//test
    Serial.println(myData2.BTN_Cut_Left2);//test
    Serial.print(" Ear: ");
    Serial.println(lv_slider_get_value(ui_Slider1));//working
    Serial.println();
    Serial.print(" Main: ");
    Serial.println(lv_slider_get_value(ui_Slider2));//working
    Serial.println();
    Serial.print(" AdjustUP: ");//test
    Serial.print("Select1: ");//test
    //Serial.println(Select1);//test


    lv_timer_handler();
    delay(5);
}